@@ -27,18 +27,18 @@ def fillnodata_upstream(idxs_ds, seq, data, nodata):
 
     Returns
     -------
-    1D array of data.dtype
+    data_out: 1D array of data.dtype
         infilled data
     """
-    res = data.copy()
+    data_out = data.copy()
     for idx0 in seq:  # down- to upstream
-        if res[idx0] == nodata:
-            res[idx0] = res[idxs_ds[idx0]]
-    return res
+        if data_out[idx0] == nodata and data_out[idxs_ds[idx0]] != nodata:
+            data_out[idx0] = data_out[idxs_ds[idx0]]
+    return data_out
 
 
 def basins(idxs_ds, idxs_pit, seq, ids=None):
-    """return basin map"""
+    """Return basin map"""
     if ids is None:
         ids = np.arange(1, idxs_pit.size + 1, dtype=np.uint32)
     basins = np.zeros(idxs_ds.size, dtype=ids.dtype)
@@ -47,8 +47,74 @@ def basins(idxs_ds, idxs_pit, seq, ids=None):
 
 
 @njit
-def subbasins(idxs_ds, seq, strord, min_sto=0):
+def interbasin_mask(idxs_ds, seq, region, stream=None):
+    """Returns most downstream contiguous area within region, i.e.: if a stream flows
+    in and out of the region, only the most downstream contiguous area within region
+    will be True in output mask. If a stream mask is provided the area is reduced to
+    cells which drain to the stream.
+
+    Parameters
+    ----------
+    idxs_ds : 1D-array of intp
+        index of next downstream cell
+    seq : 1D array of int
+        ordered cell indices from down- to upstream
+    region : 1D array of bool
+        mask of region
+    stream : 1D array of bool, optional
+        mask of stream
+
+    Returns
+    -------
+    mask: 1D array of bool
+        Mask of most downstream contiguous area within region
+    """
+    # get area upstream of streams within region
+    if stream is not None:
+        mask = stream.copy()
+        # make sure all mask contains most downstream stream cells
+        for idx0 in seq[::-1]:  # up- to downstream
+            if mask[idx0]:
+                mask[idxs_ds[idx0]] = True
+    else:
+        mask = np.array([bool(1) for _ in range(region.size)])  # all True
+    # keep only the most downstream contiguous area within region
+    for idx0 in seq:  # down- to upstream
+        idx_ds = idxs_ds[idx0]
+        mask[idx0] = mask[idx_ds]
+        if not region[idx0] and region[idx_ds]:
+            # set mask to false in first cell(s) upstream of region
+            mask[idx0] = False
+        # propagate mask upstream
+    return np.logical_and(mask, region)
+
+
+# @njit
+# def headwater_mask(idxs_ds, seq, region, stream=None):
+#     """Returns a mask of headwater subbasins within a region, i.e. basins with upstream
+#     cells outside the region are excluded. If a stream mask is provided the area is reduced
+#     to cells which drain to the stream."""
+#     # get area upstream of streams within region
+#     if stream is not None:
+#         mask = np.logical_and(region, stream)
+#         for idx in seq:  # down- to upstream
+#             mask[idx] = mask[idxs_ds[idx]]
+#     else:
+#         mask = region.copy()
+#     # keep only subbasins (areas with no upstream cells outside region)
+#     for idx in seq[::-1]:  # up- to downstream
+#         idx_ds = idxs_ds[idx]
+#         if region[idx_ds] == False:  # outside region
+#             mask[idx_ds] = False
+#         else:
+#             mask[idx_ds] = mask[idx]  # propagate mask downstream
+#     return mask
+
+
+@njit
+def subbasins_streamorder(idxs_ds, seq, strord, mask=None, min_sto=-2):
     """Returns a subbasin map with unique IDs starting from one.
+    Subbasins are defined based on a minimum stream order.
 
     Parameters
     ----------
@@ -58,6 +124,8 @@ def subbasins(idxs_ds, seq, strord, min_sto=0):
         ordered cell indices from down- to upstream
     strord : 1D-array of uint8
         stream order
+    mask : 1D array of bool, optional
+        consider only True cells
     min_sto : int, optional
         minimum stream order of subbasins, by default the stream order is set to
         two under the global maxmium stream order.
@@ -67,31 +135,23 @@ def subbasins(idxs_ds, seq, strord, min_sto=0):
     basins : 1D-arrays of uint32
         map with unique IDs for stream_order>=min_sto subbasins
     """
-    if min_sto == 0:
-        min_sto = strord.max() - 2
-    _done = np.array([np.bool(0) for _ in range(strord.size)])  # initiate False array
-    basins = np.full(idxs_ds.size, 0, dtype=np.uint32)
-    i = np.uint32(1)
+    if min_sto < 0:
+        min_sto = strord.max() + min_sto
+    subbas = np.full(idxs_ds.shape, 0, dtype=np.int32)
+    i = np.int32(1)
     for idx0 in seq[::-1]:  # up- to downstream
-        if _done[idx0] or strord[idx0] < min_sto:
+        if (mask is not None and mask[idx0] == False) or strord[idx0] < min_sto:
             continue
-        sto = strord[idx0]
-        while True:
-            idx_ds = idxs_ds[idx0]
-            sto_ds = strord[idx_ds]
-            if idx0 == idx_ds or sto_ds > sto:  # pit or new stream
-                basins[idx0] = i
-                i += 1
-                break
-            else:
-                _done[idx_ds] = True
-                idx0 = idx_ds
-    return fillnodata_upstream(idxs_ds, seq, basins, 0)
+        idx_ds = idxs_ds[idx0]
+        if strord[idx0] != strord[idx_ds] or idx_ds == idx0:
+            subbas[idx0] = i
+            i += 1
+    return fillnodata_upstream(idxs_ds, seq, subbas, 0)
 
 
 # TODO check
 # @njit # NOTE does not work atm with dicts (numba 0.48)
-def pfafstetter(idxs_pit, idxs_ds, seq, uparea, upa_min=0, depth=1, mv=_mv):
+def subbasins_pfafstetter(idxs_pit, idxs_ds, seq, uparea, upa_min=0, depth=1, mv=_mv):
     """pfafstetter subbasins coding
 
     Verdin K . and Verdin J . 1999 A topological system for delineation
