@@ -252,7 +252,7 @@ def _d8_idx(idx0, shape):
             if r_us >= 0 and r_us < nrow and c_us >= 0 and c_us < ncol:  # check bounds
                 idx = r_us * ncol + c_us
                 idxs_lst.append(idx)
-    return np.array(idxs_lst, dtype=np.intp)
+    return np.array(idxs_lst)
 
 
 @njit
@@ -382,8 +382,8 @@ def _tributaries(
     """
     if n > 0:
         # use heapq to keep n largest
-        i = np.intp(0)
-        ntrib = [(np.float64(upa_min), mv, np.intp(-1)) for _ in range(n)]
+        i = int(0)
+        ntrib = [(np.float64(upa_min), mv, int(-1)) for _ in range(n)]
         heapq.heapify(ntrib)
     else:
         idxs = []
@@ -496,3 +496,35 @@ def snap(
         idxs[i] = path[-1]
         dists[i] = d
     return idxs, dists
+
+
+@njit
+def inflow_idxs(idxs_ds, seq, region):
+    """returns linear indices of most upstream pixels within region"""
+    idxs = []
+    mask = np.array([bool(1) for _ in range(idxs_ds.size)])  # all True
+    for idx0 in seq[::-1]:  # up- to downstream
+        idx_ds = idxs_ds[idx0]
+        if idx0 != idx_ds:
+            if mask[idx0] and region[idx_ds] and not region[idx0]:  # in
+                idxs.append(idx0)
+                mask[idx_ds] = False
+            else:
+                mask[idx_ds] = mask[idx0]
+    return np.array(idxs, dtype=idxs_ds.dtype)
+
+
+@njit
+def outflow_idxs(idxs_ds, seq, region):
+    """returns linear indices of most downstream pixels within region"""
+    idxs = []
+    mask = np.array([bool(1) for _ in range(idxs_ds.size)])  # all True
+    for idx0 in seq:  # down- to upstream
+        idx_ds = idxs_ds[idx0]
+        # at mask and region and (pit or out)
+        if mask[idx_ds] and region[idx0] and (idx_ds == idx0 or not region[idx_ds]):
+            idxs.append(idx0)
+            mask[idx0] = False
+        else:
+            mask[idx0] = mask[idx_ds]
+    return np.array(idxs, dtype=idxs_ds.dtype)
