@@ -149,25 +149,40 @@ def streams(idxs_ds, seq, strord, mask=None, min_sto=1):
     streams : list of 1D-arrays of intp
         linear indices of streams
     """
-    _done = np.array([np.bool(0) for _ in range(strord.size)])  # initiate False array
-    if mask is not None:
-        _done[mask == False] = True
-    streams = []
+    # create map with confluences and list with its indices
+    idxs_conf = []
+    conf = np.array([bool(0) for _ in range(idxs_ds.size)])  # all False
     for idx0 in seq[::-1]:  # up- to downstream
-        if _done[idx0] or strord[idx0] < min_sto:
+        if strord[idx0] < (min_sto - 1):
             continue
-        idxs = [idx0]
-        sto = strord[idx0]
+        idx_ds = idxs_ds[idx0]
+        if strord[idx_ds] > strord[idx0]:
+            idxs_conf.append(idx_ds)
+            if strord[idx0] >= min_sto:
+                conf[idx_ds] = True
+    if len(idxs_conf) == 0:
+        raise ValueError("No streams found with given settings.")
+
+    # get list of indices arrays of segments
+    streams = []
+    done = np.array([bool(0) for _ in range(idxs_ds.size)])  # all False
+    for idx0 in idxs_conf:
+        if done[idx0] or (mask is not None and mask[idx0] == False):
+            continue
+        idxs = [idx0]  # initiate with correct dtype
         while True:
+            done[idx0] = True
             idx_ds = idxs_ds[idx0]
-            sto_ds = strord[idx_ds]
-            idxs.append(idx_ds)
-            if idx0 == idx_ds or sto_ds > sto:  # pit or new stream
-                streams.append(np.array(idxs, dtype=idxs_ds.dtype))
+            valid = mask is None or mask[idx_ds]
+            if valid:
+                idxs.append(idx_ds)
+            if not valid or conf[idx_ds] or done[idx_ds] or idx_ds == idx0:
+                # pit or new stream segment (only on streams with strord > min_sto)
+                if len(idxs) >= 2:
+                    streams.append(np.array(idxs, dtype=idxs_ds.dtype))
                 break
-            else:
-                _done[idx_ds] = True
-                idx0 = idx_ds
+
+            idx0 = idx_ds
     return streams
 
 
