@@ -59,31 +59,35 @@ def test_flwdirraster_errors():
     with pytest.raises(ValueError, match="Invalid FlwdirRaster: shape"):
         pyflwdir.FlwdirRaster(idxs_ds, (1, 2), "d8")
     with pytest.raises(ValueError, match="Invalid FlwdirRaster: no pits found"):
-        pyflwdir.FlwdirRaster(np.array([1, 0], dtype=np.intp), (2, 1), "d8")
+        pyflwdir.FlwdirRaster(np.array([1, 0], dtype=int), (2, 1), "d8")
 
 
 @pytest.mark.parametrize("parsed, d8", test_core.test_data)
 def test_flwdirraster_attrs(parsed, d8):
     idxs_ds, idxs_pit, seq, rank, mv = parsed
-    flw = pyflwdir.FlwdirRaster(
-        idxs_ds.copy(), d8.shape, "d8", idxs_pit=idxs_pit.copy()
-    )
-    assert flw._mv == mv
-    assert flw.size == d8.size
-    assert flw.shape == d8.shape
-    assert isinstance(flw._dict, dict)
-    assert isinstance(flw.__str__(), str)
-    assert np.all(flw[flw.idxs_pit] == flw.idxs_pit)
-    assert isinstance(flw.xy(flw.idxs_pit), tuple)
-    assert isinstance(flw.transform, Affine)
-    assert isinstance(flw.bounds, np.ndarray)
-    assert isinstance(flw.latlon, bool)
-    assert np.all(flw.rank.ravel() == rank)
-    assert flw.ncells == seq.size
-    assert np.all(np.diff(rank.flat[flw.idxs_seq]) >= 0)
-    flw.repair_loops()
-    assert flw.isvalid
-    assert np.sum(flw.mask) == flw.ncells
+    for cache in [True, False]:
+        flw = pyflwdir.FlwdirRaster(
+            idxs_ds.copy(), d8.shape, "d8", idxs_pit=idxs_pit.copy(), cache=cache
+        )
+        assert flw._mv == mv
+        assert flw.size == d8.size
+        assert flw.shape == d8.shape
+        assert isinstance(flw._dict, dict)
+        assert isinstance(flw.__str__(), str)
+        assert np.all(flw[flw.idxs_pit] == flw.idxs_pit)
+        assert isinstance(flw.xy(flw.idxs_pit), tuple)
+        assert isinstance(flw.transform, Affine)
+        assert isinstance(flw.bounds, np.ndarray)
+        assert np.allclose(flw.extent, flw.bounds[[0, 2, 1, 3]])
+        assert isinstance(flw.latlon, bool)
+        assert np.all(flw.rank.ravel() == rank)
+        if cache:
+            assert "rank" in flw._cached
+        assert flw.ncells == seq.size
+        assert np.all(np.diff(rank.flat[flw.idxs_seq]) >= 0)
+        flw.repair_loops()
+        assert flw.isvalid
+        assert np.sum(flw.mask) == flw.ncells
 
 
 def test_add_pits():
@@ -91,7 +95,7 @@ def test_add_pits():
     x, y = flw.xy(flw.idxs_pit)
     # all cells are True -> pit at idx1
     flw.order_cells()  # set flw._seq
-    flw.add_pits(idxs=idx0, streams=np.full(d8.shape, True, dtype=np.bool))
+    flw.add_pits(idxs=idx0, streams=np.full(d8.shape, True, dtype=bool))
     assert np.all(flw.idxs_pit == idx0)
     assert flw._seq is None  # check if seq is deleted
     # original pit idx0
@@ -125,7 +129,7 @@ def test_path_snap():
     assert np.all(flw.snap(xy=flw.xy(idx1), direction="up")[0] == idx0)
 
     # with mask
-    mask = np.full(flw.shape, False, dtype=np.bool)
+    mask = np.full(flw.shape, False, dtype=bool)
     path, dist = flw.path(idx0, mask=mask)
     idx2, _ = flw.snap(idx0, mask=mask)
     assert path[0].size == dist[0] + 1
@@ -152,7 +156,7 @@ def test_path_snap():
 
 
 def test_downstream():
-    idxs = np.arange(flw.size, dtype=np.intp)
+    idxs = np.arange(flw.size, dtype=int)
     assert np.all(flw.downstream(idxs).ravel()[flw.mask] == flw.idxs_ds[flw.mask])
     with pytest.raises(ValueError, match="size does not match"):
         flw.downstream(np.ones((1, 1)))
@@ -205,11 +209,11 @@ def test_basins():
 
 
 def test_subbasins():
-    pfaf = flw.pfafstetter()
+    pfaf = flw.subbasins_pfafstetter()
     bas0 = flw.basins(flw.idxs_pit[0])
     assert np.all(pfaf[bas0 != 0] > 0)
     assert pfaf.max() <= 9
-    subbas = flw.subbasins()
+    subbas = flw.subbasins_streamorder()
     assert np.all(subbas[bas0 != 0] > 0)
 
 
@@ -244,19 +248,19 @@ def test_streams():
     assert strord.dtype == np.int8
     assert np.all(strord.shape == flw.shape)
     # vectorize
-    feats = flw.features(kind="streams")
+    feats = flw.streams()
     fstrord = np.array([f["properties"]["strord"] for f in feats])
-    findex = np.array([f["properties"]["idx"] for f in feats])
+    findex = np.array([f["properties"]["idxs"] for f in feats])
     assert np.all(fstrord == strord.flat[findex])
-    feats = flw.features(kind="flwdir")
-    findex = np.array([f["properties"]["idx"] for f in feats])
+    feats = flw.vectorize()
+    findex = np.array([f["properties"]["idxs"] for f in feats])
     assert np.all(findex == np.sort(idxs_seq))
     with pytest.raises(ValueError, match="size does not match"):
-        flw.features(xs=np.arange(3), ys=np.arange(3))
+        flw.geofeatures([np.array([1, 2])], xs=np.arange(3), ys=np.arange(3))
     with pytest.raises(ValueError, match="size does not match"):
-        flw.features(mask=np.ones((1, 1)))
+        flw.streams(mask=np.ones((1, 1)))
     with pytest.raises(ValueError, match="Kwargs map"):
-        flw.features(uparea=np.ones((1, 1)))
+        flw.geofeatures([np.array([1, 2])], uparea=np.ones((1, 1)))
     # stream distance
     data = np.zeros(flw.shape, dtype=np.int32)
     data[flw.rank > 0] = 1
@@ -266,7 +270,7 @@ def test_streams():
     assert dist.dtype == np.int32
     assert np.all(dist.shape == flw.shape)
     assert np.all(dist0[dist != -9999] <= dist[dist != -9999])
-    dist = flw.stream_distance(mask=np.ones(flw.shape, dtype=np.bool))
+    dist = flw.stream_distance(mask=np.ones(flw.shape, dtype=bool))
     assert np.all(dist[dist != -9999] == 0)
     dist = flw.stream_distance(unit="m")
     assert dist.dtype == np.float64
